/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Challenges Demo                             */
/*                     Prtitioned Database Creation Script                  */
/****************************************************************************/

set noexec off
set nocount on
go

use master
go

if exists
(
	select * from sys.databases where name = 'PartitionedDB'
)
begin
	raiserror('Database PartitionedDB already created',16,1)
	set noexec on
end
go


/* 
set noexec off
go
use master 
go
if exists (select * from sys.databases where name = 'PartitionedDB')
begin
	alter database PartitionedDB set single_user with rollback immediate;
	drop database PartitionedDB;
end
*/

declare
	@version int
	,@dataPath nvarchar(512)
	,@logPath nvarchar(512) 

set @version = 
	convert(int,
		left(
			convert(nvarchar(128), serverproperty('ProductVersion')),
			charindex('.',convert(nvarchar(128), serverproperty('ProductVersion'))) - 1
		)
	)

if @version >= 11 -- SQL Server 2012+
begin
	select 
		@dataPath = convert(nvarchar(512),serverproperty('InstanceDefaultDataPath'))
		,@logPath = convert(nvarchar(512),serverproperty('InstanceDefaultLogPath'))
end
else begin
	exec master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'Software\Microsoft\MSSQLServer\MSSQLServer', N'DefaultData', @dataPath output
	exec master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'Software\Microsoft\MSSQLServer\MSSQLServer', N'DefaultLog', @logPath output
end

-- Creating database in the same folder with master
if @dataPath is null
	select @dataPath = substring(physical_name, 1, len(physical_name) - charindex('\', reverse(physical_name))) + '\'
	from master.sys.database_files 
	where file_id = 1

if @logPath is null
	select @logPath = substring(physical_name, 1, len(physical_name) - charindex('\', reverse(physical_name))) + '\'
	from master.sys.database_files 
	where file_id = 2
	
if @dataPath is null or @logPath is null
begin
	raiserror('Cannot obtain path for data and/or log file',16,1)
	set noexec on
end

if right(@dataPath, 1) <> '\'
	select @dataPath = @dataPath + '\'
if right(@logPath, 1) <> '\'
	select @logPath = @logPath + '\'
	
declare
	@SQL nvarchar(max)

select @SQL = 
	replace
	(
		replace(
N'create database [PartitionedDB]
on primary (name=N''PartitionedDB'', filename=N''%DATA%PartitionedDB.mdf'', size=10MB, filegrowth = 10MB),
filegroup [RECENTORDERS] (name=N''PartitionedDB_RecentOrders'', filename=N''%DATA%PartitionedDB_RecentOrders.ndf'', size=100MB, filegrowth = 50MB), 
filegroup [OLDORDERS] (name=N''PartitionedDB_OldOrders'', filename=N''%DATA%PartitionedDB_OldOrders.ndf'', size=100MB, filegrowth = 50MB)
log on (name=N''PartitionedDB_log'', filename=N''%LOG%PartitionedDB.ldf'', size=500MB, filegrowth = 100MB);

alter database [PartitionedDB] set recovery simple;
'
			,'%DATA%',@dataPath
		),'%LOG%',@logPath
	)

raiserror('Creating database PartitionedDB',0,1) with nowait
raiserror('Data Path: %s',0,1,@dataPath) with nowait
raiserror('Log Path: %s',0,1,@logPath) with nowait
raiserror('Statement:',0,1) with nowait
raiserror(@sql,0,1) with nowait

exec sp_executesql @sql
go

use PartitionedDB
go

create table dbo.Numbers
( Number int not null constraint PK_Numbers primary key with (data_compression=row));

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2) -- 256 rows
,N5(C) as (select 0 from N4 as T1 cross join N4 as T2 ) -- 65,536 rows
,N6(C) as (select 0 from N5 as T1 cross join N3 as T2 ) -- 1,048,576 rows
,IDs(ID) as (select row_number() over (order by (select null)) from N6)
insert into dbo.Numbers(Number)
	select ID from Ids
go
