/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Challenges Demo                             */
/*                          Statistics / Histogram                          */
/****************************************************************************/

set nocount on
go

use PartitionedDB
go

select year(OrderDate), month(OrderDate), count(*) as [Non Partitioned DB Order Count]
from NonPartitionedDb.dbo.Orders with (tablock)
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

select year(OrderDate), month(OrderDate), count(*) as [Partitioned DB Order Count]
from dbo.Orders with (tablock)
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

select year(OrderDate), month(OrderDate), count(*) as [Non Partitioned DB Order Count]
from NonPartitionedDb.dbo.Orders with (tablock)
where StoreId = 10
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

select year(OrderDate), month(OrderDate), count(*) as [Partitioned DB Order Count]
from dbo.Orders with (tablock)
where StoreId = 10
group by year(OrderDate), month(OrderDate)
order by month(OrderDate)
go

/* Statistics is up to date in every index but you can update it with:
update NonPartitionedDb.dbo.Orders with fullscan;
update statistics dbo.Orders2016_10 with fullscan;
update statistics dbo.Orders2016_09 with fullscan;
update statistics dbo.Orders2016 with fullscan;
*/

dbcc show_statistics('dbo.Orders2016_10','IDX_Orders2016_10_StoreID_OrderDate')
go

use NonPartitionedDB
go
dbcc show_statistics('dbo.Orders','IDX_Orders_StoreID_OrderDate')
go

set statistics io on
select top 100 *
from NonPartitionedDb.dbo.Orders
where StoreId = 10 and OrderDate >= '2016-10-01'
order by Amount desc
option (maxdop 1)

select top 100 *
from PartitionedDb.dbo.Orders2016_10
where StoreId = 10 and OrderDate >= '2016-10-01'
order by Amount desc
option (maxdop 1)
set statistics io off
go

dbcc show_statistics('dbo.Orders','IDX_Orders_OrderDate_OrderId')
go

use PartitionedDB
go

dbcc show_statistics('dbo.Orders2016_10','IDX_Orders2016_10_OrderDate_OrderId')
go