/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                               Data Movement                              */
/*                       Creating Non-Partitioned Table                     */
/****************************************************************************/

use PartitionedDb
go


if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'RegularTable' and s.name = 'dbo') drop table dbo.RegularTable;
if exists(select * from sys.partition_schemes where name = 'psRegularTable') drop partition scheme psRegularTable; 
if exists(select * from sys.partition_functions where name = 'pfRegularTable')	drop partition function pfRegularTable;
go


create table dbo.RegularTable
(
	OrderDate date not null,
	OrderId int not null identity(1,1),
	OrderNum varchar(32) not null,
	LobColumn varchar(max) null,
	Placeholder char(50) null,
)
TEXTIMAGE_ON [RecentOrders]
go

create unique clustered index IDX_RegularTable_OrderDate_OrderId
on dbo.RegularTable(OrderDate, OrderId)
on [RecentOrders]
go

declare
	@StartDate datetime = '2015-12-01'
	
insert into dbo.RegularTable with (tablockx) (OrderDate, OrderNum)
	select 
		dateadd(day,[days].Number ,@StartDate), 
		'Order: ' + convert(varchar(12),dateadd(day,[days].Number,@StartDate)) + '-' + convert(varchar(5),[ord].Number)
	from dbo.Numbers [days] cross join dbo.Numbers [ord]
	where [days].Number <= 395 and [ord].Number <= 512
go

select count(*) from dbo.RegularTable
go
