/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                               Data Movement                              */
/*                        Populate DB with the data                         */
/*http://aboutsqlserver.com/2016/03/08/how-to-move-data-and-log-files-online/ */
/****************************************************************************/

use PartitioningExpress
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where t.name = 'OldOrders' and s.name = 'dbo') drop table dbo.OldOrders;
go

create table dbo.OldOrders
(
	OrderDate datetime2(0) not null,
	OrderId int identity(1,1) not null,
	StoreId int not null,	
	CustomerID int not null, 
	Region char(5) not null,
	Amount money not null,
	Placeholder varchar(50) null
		constraint DEF_OldOrdersPlaceholder
		default NEWID(),

	constraint CHK_OldOrders
	check (OrderDate < '2016-10-01')
);

create unique clustered index IDX_OldOrders_OrderDate_OrderId
on dbo.OldOrders(OrderDate, OrderId)
on [FG2016]
go

create nonclustered index IDX_OldOrders_StoreID_OrderDate 
on dbo.OldOrders(StoreId, OrderDate)
on [FG2016]

declare
	@StartDate datetime = '2015-12-01'

;with N1(C) as (select 0 union all select 0) -- 2 rows
,N2(C) as (select 0 from N1 as T1 cross join N1 as T2) -- 4 rows
,N3(C) as (select 0 from N2 as T1 cross join N2 as T2) -- 16 rows
,N4(C) as (select 0 from N3 as T1 cross join N3 as T2 cross join N2 as T3) -- 1024 rows
,N5(C) as (select 0 from N4 as T1 cross join N4 ) -- 1,048,576 rows
,IDs(ID) as (select ROW_NUMBER() over (order by (select NULL)) from N5)
insert into dbo.OldOrders with (tablockx) (OrderDate,StoreId,CustomerId,Amount,Region)
	select dateadd(day,ID % 295,@StartDate), ID % 20 + 1,ID % 200000, ID % 100 + .99, convert(char(5),ID % 10000) 
	from Ids
go

select count(*) from dbo.OldOrders with (nolock)
go

