/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                    Moving Data with In-Memory OLTP                       */
/*                           Creating Tables                                */
/****************************************************************************/
set nocount on
set xact_abort on
go

/* This script requires SQL Server 2016. You can remove CHECK and FK constraints in 
memory-optimized tables to make it compatible with SQL Server 2014 */

use InMemoryOLTP2016
go


if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'InsertPayments2016_10') drop proc dbo.InsertPayments2016_10;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'UpdatePayments2016_10') drop proc dbo.UpdatePayments2016_10;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'DeletePayments2016_10') drop proc dbo.DeletePayments2016_10;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'UpdatePayments2016') drop proc dbo.UpdatePayments2016;
if exists(select * from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id where s.name = 'dbo' and p.name = 'DeletePayments2016') drop proc dbo.DeletePayments2016;
if exists(select * from sys.views v join sys.schemas s on v.schema_id = s.schema_id where s.name = 'dbo' and v.name = 'Payments') drop view dbo.Payments;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Payments2016_10') drop table dbo.Payments2016_10;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Payments2016_09') drop table dbo.Payments2016_09;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Payments2016') drop table dbo.Payments2016;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Payments2015') drop table dbo.Payments2015;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'Customers') drop table dbo.Customers;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'PaymentsUpdateQueue') drop table dbo.PaymentsUpdateQueue;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name = 'PaymentsDeleteQueue') drop table dbo.PaymentsDeleteQueue;
if exists(select * from sys.partition_schemes where name = 'ps2015') drop partition scheme ps2015;
if exists(select * from sys.partition_schemes where name = 'ps2016') drop partition scheme ps2016;
if exists(select * from sys.partition_functions where name = 'pf2015') drop partition function pf2015;
if exists(select * from sys.partition_functions where name = 'pf2016') drop partition function pf2016;
go

-- Catalog Entity
create table dbo.Customers
(
	CustomerId int not null
		constraint PK_Customers
		primary key nonclustered hash
		with (bucket_count=65536),
	Name nvarchar(256) not null, 

	index IDX_Customers_Name
	nonclustered(Name)
)
with (memory_optimized=on, durability=schema_and_data)
go

-- Storing data for 2016_10
create table dbo.Payments2016_10
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	/* Other columns */
	constraint PK_Payments2016_10
	primary key nonclustered hash(PaymentId)
	with (bucket_count=1048576),
	
	index IDX_Payments2016_10_CustomerId
	nonclustered hash(CustomerId)
	with (bucket_count=65536),

	constraint CHK_Payments2016_10
	check (PaymentDate >= '2016-10-01' and PaymentDate < '2016-11-01'),

	constraint FK_Payments2016_10_Customers
	foreign key(CustomerId)
	references dbo.Customers(CustomerId)
)
with (memory_optimized=on, durability=schema_and_data)
go

-- Storing data for 2016_09
create table dbo.Payments2016_09
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	/* Other columns */
	constraint PK_Payments2016_09
	primary key nonclustered hash(PaymentId)
	with (bucket_count=1048576),
	
	index IDX_Payments2016_09_CustomerId
	nonclustered hash(CustomerId)
	with (bucket_count=65536),

	constraint CHK_Payments2016_09
	check (PaymentDate >= '2016-09-01' and PaymentDate < '2016-10-01'),

	constraint FK_Payments2016_09_Customers
	foreign key(CustomerId)
	references dbo.Customers(CustomerId)
)
with (memory_optimized=on, durability=schema_and_data)
go





create partition function pf2016(datetime2(0))
as range right for values 
('2016-02-01','2016-03-01','2016-04-01','2016-05-01','2016-06-01','2016-07-01'
,'2016-08-01','2016-09-01','2016-10-01','2016-11-01','2016-12-01','2017-01-01');
go

create partition scheme ps2016
as partition pf2016
all to ([FG2016]);
go

-- Storing data for 2016
create table dbo.Payments2016
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null, 
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	constraint CHK_Payment2016_01_09 check (PaymentDate >= '2016-01-01' and PaymentDate < '2016-09-01'),
	constraint CHK_Payment2016_01_10 check (PaymentDate >= '2016-01-01' and PaymentDate < '2016-10-01'),
	constraint CHK_Payment2016_01_11 check (PaymentDate >= '2016-01-01' and PaymentDate < '2016-11-01'),
	constraint CHK_Payment2016_01_12 check (PaymentDate >= '2016-01-01' and PaymentDate < '2016-12-01'),
	constraint CHK_Payment2016 check (PaymentDate >= '2016-01-01' and PaymentDate < '2017-01-01'),
)
go

create unique clustered index IDX_Payments2016_PaymentDate_PaymentId
on dbo.Payments2016(PaymentDate, PaymentId)
with (data_compression=row)
on ps2016(PaymentDate)
go

create nonclustered index IDX_Payments2016_CustomerId
on  dbo.Payments2016(CustomerId)
with (data_compression=row)
on ps2016(PaymentDate)
go

create nonclustered index IDX_Payments2016_PaymentId
on  dbo.Payments2016(PaymentId)
with (data_compression=row)
on ps2016(PaymentDate)
go


create partition function pf2015(datetime2(0))
as range right for values 
('2015-02-01','2015-03-01','2015-04-01','2015-05-01','2015-06-01','2015-07-01'
,'2015-08-01','2015-09-01','2015-10-01','2015-11-01','2015-12-01','2016-01-01');
go

create partition scheme ps2015
as partition pf2015
all to ([FG2015]);
go

create table dbo.Payments2015
(
	PaymentDate datetime2(0) not null,
	PaymentId uniqueidentifier not null,
	CustomerId int not null,
	Amount money not null,
	Status tinyint not null,

	constraint CHK_Payment2015 check (PaymentDate >= '2015-01-01' and PaymentDate < '2016-01-01'),
)
on ps2015(PaymentDate)
go

create clustered columnstore index CCI_Payments2015
on dbo.Payments2015
on ps2015(PaymentDate)
go

-- Just to demo.. 
create nonclustered index IDX_Payments2015_CustomerId
on  dbo.Payments2015(CustomerId)
include(Amount)
with (data_compression=row)
on ps2015(PaymentDate)
go

create view dbo.Payments(PaymentDate, PaymentId, CustomerId, Amount, Status)
as
	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2016_10
	
	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2016_09

	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2016

	union all

	select PaymentDate, PaymentId, CustomerId, Amount, Status
	from dbo.Payments2015
go


/* Populating data */
insert into dbo.Customers(CustomerId,Name)
	select Num, 'Customer ' + convert(varchar(6),Num)
	from dbo.Numbers
	where Num <= 16384;

insert into dbo.Payments2016_10(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(second,Num * 39,'2016-10-01') 
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;

insert into dbo.Payments2016_09(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(second,Num * 39,'2016-09-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;

insert into dbo.Payments2016(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(minute,Num * 2,'2016-01-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 175000;

insert into dbo.Payments2015(PaymentDate, PaymentId, CustomerId, Amount, Status)
	select 
		dateadd(minute,Num * 7,'2015-01-01')
		,NewId()
		,Num % 16384 + 1
		,10000.00 * rand()
		,1
	from dbo.Numbers
	where Num <= 65536;
go

alter index CCI_Payments2015 on dbo.Payments2015 rebuild;
go

/* Execution Plans */
select count(*)
from dbo.Payments 
where PaymentDate between '2016-06-02' and '2016-06-03';

select count(*)
from dbo.Payments 
where PaymentDate >= '2016-01-01';

select count(*)
from dbo.Payments 
go

